//
//  AccessTokenViewController.swift
//  PresenceSDKDemoIntegration
//
//  Created by Vladislav Grigoryev on 25/10/2018.
//  Copyright © 2018 Ticketmaster. All rights reserved.
//

import Foundation
import UIKit
import PresenceSDK // for PSDK.shared

class AccessTokenViewController: UIViewController {
    
    @IBOutlet var tableView: UITableView!
    
    @IBOutlet var getButton: UIButton!
    
    var backendName = PSDK.BackendName.host {
        didSet {
            guard oldValue != backendName, isViewLoaded else {
                return
            }
            reload()
        }
    }
    
    fileprivate var info: (accessToken: String?, error: Error?) = (nil, nil) {
        didSet {
            reload()
        }
    }
    
    fileprivate var items = [(title: String, value: String)]() {
        didSet {
            tableView.reloadData()
        }
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        tableView.dataSource = self
        tableView.delegate = self
        reload()
    }
    
    func reload() {
        switch backendName {
        case .host:
            title = "Host Access Token"
            getButton.setTitle("Get Host Access Token", for: .normal)
        case .accountManager:
            title = "AccountManager Access Token"
            getButton.setTitle("Get AccountManager Access Token", for: .normal)
        case .sportXR:
            title = "SportXR Access Token"
            getButton.setTitle("Get SportXR Access Token", for: .normal)
        @unknown default:
            title = "UnknownBackend Access Token"
            getButton.setTitle("Get UnknownBackend Access Token", for: .normal)
        }
        
        let null = "null"
        items = [
            ("Access Token", info.accessToken ?? null),
            ("Error", info.error?.localizedDescription ?? null)
        ]
    }
    
    @IBAction func login(_ sender: Any) {
        // try to get access token, present login UI if needed
        PSDK.shared.getAccessToken(backendName: backendName,
                                   presentLoginUI: true) { [weak self] (accessToken) in
            self?.info = (accessToken, nil)
        } failure: { [weak self] (error, canceled) in
            self?.info = (nil, error ?? (canceled ? "Canceled" : nil))
        }
    }
    
    @IBAction func getAccessToken(_ sender: Any) {
        // try to get access token, but fail if not logged in
        PSDK.shared.getAccessToken(backendName: backendName,
                                   presentLoginUI: false) { [weak self] (accessToken) in
            self?.info = (accessToken, nil)
        } failure: { [weak self] (error, canceled) in
            self?.info = (nil, error ?? (canceled ? "Canceled" : nil))
        }
    }
    
    @IBAction func logout(_ sender: Any) {
        switch backendName {
        case .host:
            PSDK.shared.logOutHost(
                success: {
                    print("Logout Host success")
                },
                failure: { error in
                    print("Logout Host failure \(error)")
                }
            )
        case .accountManager:
            PSDK.shared.logOutTeam(
                success: {
                    print("Logout Team success")
                },
                failure: { error in
                    print("Logout Team failure \(error)")
                }
            )
        case .sportXR:
            PSDK.shared.logOutSportXR(success: {
                print("Logout SportXR success")
            }, failure: { error in
                print("Logout SportXR failure \(error)")
            })
        @unknown default:
            break
        }
        info = (nil, nil)
    }
}

extension AccessTokenViewController: UITableViewDataSource {
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return items.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "Cell") ?? UITableViewCell(style: .value1, reuseIdentifier: "Cell")
        let item = items[indexPath.row]
        cell.selectionStyle = .none
        cell.textLabel?.text = item.title
        cell.detailTextLabel?.text = item.value
        return cell
    }
}

extension AccessTokenViewController: UITableViewDelegate {
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        UIPasteboard.general.string = items[indexPath.row].value
    }
}

// MARK: - DemoIntegration Helpers -

class ShowHostAccessTokenSegue: UIStoryboardSegue {
    
    override func perform() {
        (destination as? AccessTokenViewController)?.backendName = .host
        super.perform()
    }
}

class ShowTeamAccessTokenSegue: UIStoryboardSegue {
    
    override func perform() {
        (destination as? AccessTokenViewController)?.backendName = .accountManager
        super.perform()
    }
}

class ShowSportXRAccessTokenSegue: UIStoryboardSegue {
    
    override func perform() {
        (destination as? AccessTokenViewController)?.backendName = .sportXR
        super.perform()
    }
}
