//
//  PresenceManager+LoginDelegate.swift
//  PresenceSDKDemoIntegration
//
//  Created by Jonathan Backer on 10/11/21.
//  Copyright © 2021 Ticketmaster. All rights reserved.
//

import Foundation
import PresenceSDK // for PresenceLoginDelegate

extension PresenceManager: PresenceLoginDelegate {
    
    // all methods are optional

    // MARK: Login
    
    /// Method is invoked when the LoginWindow is made visible to the user.
    func onLoginWindowDidDisplay(backendName: PSDK.BackendName) {
        print("onLoginWindowDidDisplay: \(backendName.description)")
    }

    /// Method is invoked if the user clicks on "forgot password" link.
    /// - parameter backendName: Name of the backend this callback event is associated with.
    func onLoginForgotPasswordClicked(backendName: PSDK.BackendName) {
        print("onLoginForgotPasswordClicked: \(backendName.description)")
    }

    /// Method is invoked when a login method was used to get an access token.
    /// - parameter backendName: Name of the backend this callback event is associated with.
    /// - parameter loginMethod: Name of methods used to log in.
    func onLoginMethodUsed(backendName: PSDK.BackendName, loginMethod: PSDK.LoginMethod) {
        print("onLoginMethodUsed: \(backendName.description) method: \(loginMethod.description)")
    }
    
    /// Method is invoked if the user granted app access/logged in.
    /// - parameter backendName: Name of the backend this callback event is associated with.
    func onLoginSuccessful(backendName: PSDK.BackendName, accessToken: String) {
        print("onLoginSuccessful: \(backendName.description)")
    }

    /// Method is invoked if the user dismissed login window via the Cancel button.
    func onLoginCancelled(backendName: PSDK.BackendName) {
        print("onLoginCancelled: \(backendName.description)")
    }
    
    /// Method is invoked if the backend service fails with an Error
    /// - parameter backendName: Name of the backend this callback event is associated with.
    /// - parameter error: If available, an `NSError` object is returned. Defaults is `nil`.
    func onLoginFailed(backendName: PSDK.BackendName, error: NSError) {
        print("onLoginFailed: \(backendName.description) error: \(error.localizedDescription)")
    }
    
    
    
    // MARK: Refresh Token
        
    /// Notify whenever SDK refreshes the access token.
    func onTokenRefreshed(backendName: PSDK.BackendName, accessToken: String) {
        print("onTokenRefreshed: \(backendName.description)")
        // The OAuth Token usually lasts 2+ hours
    }

    /// Notify when SDK is not able to refresh access token.
    func onRefreshTokenFailed(backendName: PSDK.BackendName) {
        print("onRefreshTokenFailed: \(backendName.description)")
    }
    
    /// Method is invoked when the refresh token has expired, the user will be forced to login again to aquire a new refresh token
    /// - parameter backendName: Name of the backend this callback event is associated with.
    func onRefreshTokenExpired(backendName: PSDK.BackendName) {
        print("onRefreshTokenExpired: \(backendName.description)")
        // The Refresh Token usually lasts 30+ days
    }
    
    

    // MARK: Logout
    
    /// Method is invoked when user has successfully logged-out
    /// - parameter backendName: Name of the backend this callback event is associated with.
    func onLogoutSuccessful(backendName: PSDK.BackendName) {
        print("onLogoutSuccessful: \(backendName.description)")
    }

    /// Method is invoked when backend service fails to logout (as part of OAuth2 spec)
    /// - parameter backendName: Name of the backend this callback event is associated with.
    /// - parameter error: Error describing the problem
    func onLogoutFailed(backendName: PSDK.BackendName, error: Error) {
        print("onLoginFailed: \(backendName.description) error: \(error.localizedDescription)")
    }

    /// Method is invoked when user successfully logged-out from all backends
    func onLogoutAllSuccessful() {
        print("onLogoutAllSuccessful")
    }
    
    
    
    // MARK: Member Info
        
    /// Called when results are returned for a Member info request after login or after updates
    /// - parameter backendName: Name of the backend this callback event is associated with.
    /// - parameter member: Member object.
    func onMemberUpdated(backendName: PSDK.BackendName, member: PresenceMember) {
        print("onMemberUpdated: \(member.email)")
        switch backendName {
        case .host:
            print("        Host Id: \(member.id)")
        case .accountManager:
            print("        Team Id: \(member.id)")
        case .sportXR:
            print("     SportXR Id: \(member.id)")
        @unknown default:
            print("     Unknown Id: \(member.id)")
        }
    }

    
    
    // MARK: Cached Info

    /// Notify when all cache is cleared
    func onCacheCleared() {
        print("onCacheCleared")
    }
}
